/*	AMX Mod X script.

	Common functions include by Lev.

	Version 1.1
*/

#if defined _common_functions_in
  #endinput
#endif
#define _common_functions_in

#include <fakemeta>

#pragma ctrlchar '\\'

/// Outputs long text in client or server console.
stock PrintLongText(id, text[])
{
	new maxLineLen = 79;
	new len = strlen(text);
	// Print lines while left size is more than maxLineLen
	new pos, lineEnd, lineLen, c;
	while (pos + maxLineLen < len)
	{
		lineEnd = strfind(text[pos], "\n");
		lineLen = lineEnd >= 0 && lineEnd < maxLineLen ? lineEnd : maxLineLen;
		lineEnd = pos + lineLen;
		c = text[lineEnd];
		text[lineEnd] = 0;
		console_print(id, "%s", text[pos]);
		text[lineEnd] = c;
		pos = lineEnd;
		if (c == '\n') pos++;
		if (text[pos] == '\r') pos++;
	}
    // Print last line
	console_print(id, "%s", text[pos]);
}

/// Converts string of flags into int.
/// Only chars from 'a' to 'z' are converted, others are skipped.
stock read_flags_fixed(const flags[])
{
	new result, i = 0;
	while (flags[i] != 0)
	{
		if (flags[i] >= 'a' && flags[i] <= 'z')
			result |= 1 << (flags[i] - 'a');
		i++;	
	}
	return result;
}

/// Compare two integes as unsigned.
/// Retruns:
/// -1 if first is smaller than second;
///  0 if first is equal to second;
///  1 if first is greater than second.
stock CompareUnsigned(first, second)
{
	if (first == second)
		return 0;
	new bool:highFirst, bool:highSecond;
	if (first < 0)
	{
		highFirst = true;
		first = first & (~(1 << 31));
	}
	if (second < 0)
	{
		highSecond = true;
		second = second & (~(1 << 31));
	}
	if (highFirst && !highSecond)
		return 1;
	if (!highFirst && highSecond)
		return -1;
	if (first > second)
		return 1;
	return -1;
}

/// Plays given sound for all players excluding bots.
stock PlaySoundForAll(const snd[], excludePlayer)
{
	if (snd[0] == 0)
		return;
	new players[32], num;
	get_players(players, num);
	for (new i = 0; i < num; i++)
	{
		new id = players[i];
		if (!is_user_bot(id) && id != excludePlayer)
			client_cmd(id, "spk %s", snd);
	}
}

/* Flags for LocatePlayers function:
*  1  - obey immunity
*  2  - allow yourself even if has immunity
*  4  - must be alive
*  8  - can't be bot
*  16 - can't be HLTV */
enum LocatePlayerFlags
{
	LP_NONE				= 0,
	LP_OBEY_IMMUNITY	= (1<<0),
	LP_ALLOW_SELF		= (1<<1),
	LP_ONLY_ALIVE		= (1<<2),
	LP_NO_BOTS			= (1<<3),
	LP_NO_HLTV			= (1<<4),
}

/// Finds player index based on Steam ID, partial player name, user IP or user ID.
/// If player not found or multiple players found returns -1.
/// If found player doesn't match options given via flags then function returns -1.
/// On success returns player index.
/// Remark: this function outputs information in console if player is not located.
stock LocatePlayer(id, const identStr[], const pluginTag[], LocatePlayerFlags:flags = LP_OBEY_IMMUNITY)
{
	new player, player1[4], player2[4];
	// Find based on steam ID
	player1[0] = find_player("c", identStr);
	player2[0] = find_player("cj", identStr);

	// Find based on a partial non-case sensitive name
	player1[1] = find_player("bl", identStr);
	player2[1] = find_player("blj", identStr);

	// Find based on IP address
	player1[2] = find_player("d", identStr);
	player2[2] = find_player("dj", identStr);

	// Find based on user ID
	if (identStr[0]=='#' && identStr[1])
		player1[3] = player2[3] = find_player("k", str_to_num(identStr[1]));

	// Check if multiple players found
	for (new i = 0; i < 4; i++)
	{
		if (player1[i] != 0)
		{
			if ((player != 0 && player != player1[i]) || player1[i] != player2[i])
			{
				console_print(id, "[%s] %L.", pluginTag, id, "MORE_CL_MATCHT", identStr);
				return -1;
			}
			player = player1[i];
		}
	}

	// Check if player not found
	if (!player)
	{
		console_print(id, "[%s] %L.", pluginTag, id, "CL_NOT_FOUND");
		return -1;
	}

	// Check for immunity (ignore immunity if found player is himself and flag LP_ALLOW_SELF set)
	if ((flags & LP_OBEY_IMMUNITY) && (get_user_flags(player) & ADMIN_IMMUNITY) && 
		((flags & LP_ALLOW_SELF) != LP_ALLOW_SELF || id != player))
	{
		new name[32];
		get_user_name(player, name, charsmax(name));
		console_print(id, "[%s] %L.", pluginTag, id, "CLIENT_IMM", name);
		return -1;
	}

	// Check if player is alive
	if ((flags & LP_ONLY_ALIVE) && !is_user_alive(player))
	{
		new name[32];
		get_user_name(player, name, charsmax(name));
		console_print(id, "[%s] %L.", pluginTag, id, "CANT_PERF_DEAD", name);
		return -1;
	}

	// Check for a bot
	if ((flags & LP_NO_BOTS) && is_user_bot(player))
	{
		new name[32];
		get_user_name(player, name, charsmax(name));
		console_print(id, "[%s] %L.", pluginTag, id, "CANT_PERF_BOT", name);
		return -1;
	}

	// Check for a HLTV
	if ((flags & LP_NO_HLTV) && (pev(player, pev_flags) & FL_PROXY) == FL_PROXY)
	{
		new name[32];
		get_user_name(player, name, charsmax(name));
		console_print(id, "[%s] That action can't be performed on HLTV \"%s\".", pluginTag, name);
		return -1;
	}

	return player;
}
